import os
import sys
import time
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *


t=time.time()
mTime = [[t for x in range(4)] for y in range(8)]
mLast = [[0 for x in range(4)] for y in range(8)]
MTHROTTLE=0.05

def getMOTION(addr,chan):
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=4)),"Motion sensor value out of range. Must have a value of 1 to 4"
    #VerifyCHAN(chan)
    chan = chan - 1
    if ((time.time()-mTime[addr][chan]) >= MTHROTTLE):
        mTime[addr][chan]=time.time()
        resp=ppCMD(addr,0x30,chan,0,2)
        val=(256*resp[0]+resp[1])
        val=(val*5.10*2.4/4095)
        if (val>=2.5):
            mLast[addr][chan]=1
        else:
            mLast[addr][chan]=0
    return mLast[addr][chan]
    
def getPOT(addr,chan,range=None):
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=4)),"Pot channel out of range. Must have a value of 1 to 4"
    MAX=5.0
    if (range != None):
        assert((range>0.0) and (range<=12.0)),"The range argument must be between 0 and 12 volts."
        MAX=range
    resp=ppCMD(addr,0x30,chan-1,0,2)
    val=(256*resp[0]+resp[1])
    val=(val*5.10*2.4/4095)
    arg=round((100*val/MAX),3)
    if (arg>100.0):
        arg=100.0
    return arg
    
    


#==============================================================================#	
# BASIC I/O Functions	                                                       #
#==============================================================================#          
import time
import spidev
import RPi.GPIO as GPIO

baseADDR=48
ppNAME='TINKERplate'

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
baseADDR=48
MAXADDR=8
ppFRAME = 25
ppINT = 22
ppACK = 23
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #pause to let Pi-Plate reset SPI HW if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")

DataGood=False
platesPresent = list(range(8))

def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),ppNAME+" address out of range"
    addr_str=str(addr)
    assert (platesPresent[addr]==1),"No "+ppNAME+" found at address "+addr_str
    
def VerifyCHAN(chan):
    assert ((chan>=1) and (chan<9)),"Invalid channel number - must be between 1 and 8"
  
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global baseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr+baseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    t0=time.time()
    wait=True    
    while(wait):
        if (GPIO.input(ppACK)==1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        ppFRAME = 25    
        GPIO.output(ppFRAME,True)
        null=spi.xfer(arg,500000,5)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False    
        if (bytes2return>0) and DataGood:
            t0=time.time()
            wait=True
            while(wait):
                if (GPIO.input(ppACK)!=1):              
                    wait=False
                if ((time.time()-t0)>0.08):   #timeout
                    wait=False
                    DataGood=False
            if (DataGood==True):
                for i in range(0,bytes2return+1):	
                    dummy=spi.xfer([00],500000,5)
                    resp.append(dummy[0])
                csum=0;
                for i in range(0,bytes2return):
                    csum+=resp[i]
                if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                    DataGood=False
        GPIO.output(ppFRAME,False)
    return resp

def getID(addr):
    global baseADDR
    VerifyADDR(addr)
    addr=addr+baseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True): 
        ppFRAME = 25
        GPIO.output(ppFRAME,True)
        null=spi.xfer(arg,500000,50)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,40)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,40)  
                    checkSum=dummy[0]                
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            #print checkSum, ~checkSum & 0xFF, csum & 0xFF
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
    return id   
    
def getADDR(addr):
    global baseADDR
    resp=ppCMD(addr,0x00,0,0,1)
    #print resp, DataGood;
    if (DataGood):
        return resp[0]-baseADDR
    else:
        return 8
    
def quietPoll():   
    global platesPresent
    ppFoundCount=0
    for i in range (0,8):
        platesPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            platesPresent[i]=1
            ppFoundCount += 1
        
def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)
    time.sleep(.10)


#import spidev
import time
import string
import site
import sys
import math
from numbers import Number
import os

file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)

from BASE import *
from A2D import *
from LED import *
from RELAY import *
from DIO import *
from PWM import *
from SERVO import *
from RANGE import *
from TEMP import *
from METER import *
from ANFUNC import *
from BUTTON import *

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("This library is only compatible with Python 3.")
    exit(1)  

localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/TINKERhelp.txt'
#helpPath='TINKERhelp.txt'       #for development only
TINKERversion=1.0
#DataGood=False

dModes=['din','dout','button','pwm','range','temp','servo','rgbled','motion']
pcaRequired=[0,0,0,1,0,0,0,0,0]
chanModes=['din','din','din','din','din','din','din','din']
pcaMap=[0,1,2,3,4,5,6,6]

#RMAX = 2000

def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

#==============================================================================#	
# setMODE - configure digital channels. Uses command in 0x9X range 	           #
# Valid modes are:
#    DIN
#    DOUT
#    BUTTON
#    PWM
#    RANGE
#    TEMP
#    SERVO
#    RGBLED
#==============================================================================# 
def setMODE(addr, chan, mode):
    modeCount=9
    selMode=modeCount
    VerifyADDR(addr)
    if (mode != 'range'):
        VerifyCHAN(chan)
        chan-=1
    else:
        channelpair=chan
        assert ((channelpair==12) or (channelpair==34) or (channelpair==56) or (channelpair==78)),"Invalid channel pair argument. Valid values are 12, 34, 56, and 78"      
        chan=(channelpair>>1)//10
    mode=mode.lower()
    if(mode=='led'):
        mode='pwm'
    for i in range(modeCount):
        if (mode==dModes[i]):
            selMode=i
    assert (selMode!=9), mode+' is not a valid mode.'
    if (pcaRequired[selMode]==1):
        assert (pcaMap[chan]!=6), 'This channel can not support '+mode+' mode.'
    # if (mode=='range'):
        # chan=(chan>>1)<<1  #convert odd number channel selection for rangefinder to lower even number.
    resp=ppCMD(addr,0x90,chan,selMode,0)
    
def setDEFAULTS(addr):   #Use this function to set all the ports back to inputs 
    VerifyADDR(addr) 
    for i in range(8):
        resp=ppCMD(addr,0x90,i,0,0)
   
#==============================================================================#	
# System Functions	                                                           #
#==============================================================================#
def getFWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0
	
def getHWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	

def getVersion():
    return TINKERversion   

def setINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF5,0,0,0)

quietPoll()



import spidev
import time
import string
import site
import sys
import RPi.GPIO as GPIO
from six.moves import input as raw_input
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (2,7,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this module")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
GPIObaseADDR=8
ppFRAME = 25
ppINT = 22
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)    #reserve interrupt input
spi = spidev.SpiDev()
spi.open(0,1)	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/DAQChelp.txt'
#helpPath='DAQChelp.txt'
DAQCversion=1.5
#Version 1.5 - fixed read issues with getaADCall and getID
#Version 1.4 - added Python 3 compatibility
daqcsPresent = list(range(8))
Vcc=list(range(8))
MAXADDR=8
	
def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")



#===============================================================================#	
# ADC Functions	     	                                              			#
#===============================================================================#	
def getADC(addr,channel):
    VerifyADDR(addr)
    VerifyAINchannel(channel)
	
    resp=ppCMD(addr,0x30,channel,0,2)
    value=(256*resp[0]+resp[1])
    value=round(value*4.096/1024,3)
    if (channel==8):
        value=value*2.0
    return value

def getADCall(addr):
    value=range(8)
    VerifyADDR(addr)    
    #resp=ppCMD(addr,0x31,0,0,16)
    for i in range (0,8):
        resp=ppCMD(addr,0x30,i,0,2)
        value[i]=(256*resp[0]+resp[1])
        value[i]=round(value[i]*4.096/1024,3)        
        # value[i]=(256*resp[2*i]+resp[2*i+1])
        # value[i]=round(value[i]*4.096/1024,3)
    return value    
    
#===============================================================================#	
# Digital Input Functions	                                                   	#
#===============================================================================#
def getDINbit(addr,bit):
    VerifyADDR(addr)
    VerifyDINchannel(bit)
    resp=ppCMD(addr,0x20,bit,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
		
def getDINall(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]

def enableDINint(addr, bit, edge):	# enable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit)	
    if ((edge=='f') or (edge=='F')):
        resp=ppCMD(addr,0x21,bit,0,0)		
    if ((edge=='r') or (edge=='R')):
        resp=ppCMD(addr,0x22,bit,0,0)
    if ((edge=='b') or (edge=='B')):
        resp=ppCMD(addr,0x23,bit,0,0)		
		
def disableDINint(addr,bit):	# disable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit)
    resp=ppCMD(addr,0x24,bit,0,0)
	
def getTEMP(addr,channel,scale):
    VerifyADDR(addr)
    assert ((channel>=0) and (channel<=7)),"Channel value out of range. Must be a value between 0 and 7"
    scal=scale.lower()
    assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    resp=ppCMD(addr,0x70,channel,0,0)   #initiate measurement
    time.sleep(1)
    resp=ppCMD(addr,0x71,channel,0,2)   #get data
    Temp=resp[0]*256+resp[1]
    if (Temp>0x8000):
        Temp = Temp^0xFFFF
        Temp = -(Temp+1)
    Temp = round((Temp/16.0),4)
    if (scal=='k'):
        Temp = Temp + 273
    if (scal=='f'):
        Temp = round((Temp*1.8+32.2),4)
    return Temp
    
    
#===============================================================================#	
# Hybrid Functions	                                                   	#
#===============================================================================#    
def getRANGE(addr,channel,units):
    VerifyADDR(addr)
    assert ((channel>=0) and (channel<=6)),"Channel value out of range. Must be a value between 0 and 6"
    uni=units.lower()
    assert ((uni=='c') or (uni=='i')), "ERROR: incorrect units parameter. Must be 'c' or 'i'."
    resp=ppCMD(addr,0x80,channel,0,0)   #initiate measurement
    time.sleep(.07)
    resp=ppCMD(addr,0x81,channel,0,2)   #get data
    Range=resp[0]*256+resp[1]
    assert (Range!=0), "ERROR: sensor failure"
    if (uni=='c'):
        Range = Range/58.326
    if (uni=='i'):
        Range = Range/148.148
    Range=round(Range,2)
    return Range
    
#===============================================================================#	
# LED Functions	                                                   		   		#
#===============================================================================#			
def setLED(addr,led):
    VerifyADDR(addr)
    VerifyLED(led)
    resp=ppCMD(addr,0x60,led,0,0)

def clrLED(addr,led):
    VerifyADDR(addr)
    VerifyLED(led)
    resp=ppCMD(addr,0x61,led,0,0)

def toggleLED(addr,led):
    VerifyADDR(addr)
    VerifyLED(led)
    resp=ppCMD(addr,0x62,led,0,0)	

def getLED(addr,led):
    VerifyADDR(addr)
    VerifyLED(led)
    resp=ppCMD(addr,0x63,led,0,1)
    return resp[0]	

def VerifyLED(led):
    assert (led>=0 and led<=1),"Invalid LED value. Must be 0 or 1"
    
#==============================================================================#	
# Switch Functions	                                                   #
#==============================================================================#		
def getSWstate(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x50,0,0,1)
    return resp[0]

def enableSWint(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x51,0,0,0)

def disableSWint(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x52,0,0,0)		
	
def enableSWpower(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x53,0,0,0)

def disableSWpower(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x54,0,0,0)
		
		
#==============================================================================#	
# Digital Output Functions	                                                   #
#==============================================================================#	
def setDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x10,bit,0,0)

	
def clrDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x11,bit,0,0)		

def toggleDOUTbit(addr,bit):
    VerifyADDR(addr)
    VerifyDOUTchannel(bit)
    resp=ppCMD(addr,0x12,bit,0,0)		
	
def setDOUTall(addr,byte):
    VerifyADDR(addr)
    assert ((byte>=0) and (byte<=127)),"Digital output value out of range. Must be in the range of 0 to 127"
    resp=ppCMD(addr,0x13,byte,0,0)			

def getDOUTbyte(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x14,0,0,1)
    return resp


#==============================================================================#	
# PWM and DAC Output Functions	                                                   #
#==============================================================================#	
def setPWM(addr,channel,value):
    VerifyADDR(addr)
    assert (value<=1023 and value>=0), "ERROR: PWM argument out of range - must be between 0 and 1023"
    assert (channel==0 or channel==1), "Error: PWM channel must be 0 or 1"
    hibyte = value>>8
    lobyte = value - (hibyte<<8)
    resp=ppCMD(addr,0x40+channel,hibyte,lobyte,0)

def getPWM(addr,channel):
    VerifyADDR(addr)
    assert (channel==0 or channel==1), "Error: PWM channel must be 0 or 1"
    ## Return PWM set value
    resp=ppCMD(addr,0x40+channel+2,0,0,2)
    value=(256*resp[0]+resp[1])
    return value	
	
def setDAC(addr,channel,value):
    global Vcc
    VerifyADDR(addr)
    assert (value>=0 and value<=4.095), "ERROR: PWM argument out of range - must be between 0 and 4.095 volts"
    assert (channel==0 or channel==1), "Error: DAC channel must be 0 or 1"
    value = int(value/Vcc[addr]*1024)
    hibyte = value>>8
    lobyte = value - (hibyte<<8)
    resp=ppCMD(addr,0x40+channel,hibyte,lobyte,0)

	
def getDAC(addr,channel):
    global Vcc
    VerifyADDR(addr)
    assert (channel==0 or channel==1), "Error: DAC channel must be 0 or 1"
    ## Return DAC value
    resp=ppCMD(addr,0x40+channel+2,0,0,2)
    value=(256*resp[0]+resp[1])
    value=value*Vcc[addr]/1023
    return value

def calDAC(addr):
    global Vcc
    VerifyADDR(addr)
    Vcc[addr] = getADC(addr,8)   
        
#==============================================================================#	
# Interrupt Control Functions	                                               #
#==============================================================================#	
def intEnable(addr):	#DAQC will pull down on INT pin if an enabled event occurs
    VerifyADDR(addr)
    resp=ppCMD(addr,0x04,0,0,0)
	
def intDisable(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x05,0,0,0)
	
def getINTflags(addr):	#read INT flag registers in DAQC
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,2)
    value=(256*resp[0]+resp[1])
    return value
		
#==============================================================================#	
# System Functions	                                                   		   #
#==============================================================================#	
def getFWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0
	
def getHWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	

def getVersion():
    return DAQCversion    
    
def getADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"DAQCplate address must be in the range of 0 to 7"
    resp=ppCMD(addr,0x00,0,0,1)
    return resp[0]
	
def getID(addr):
    global GPIObaseADDR
    VerifyADDR(addr)
    addr=addr+GPIObaseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME = 25
    GPIO.output(ppFRAME,True)
    #null = spi.writebytes(arg)
    null=spi.xfer(arg,300000,60)
    count=0
#    time.sleep(.0001)
    while (count<20): 
        dummy=spi.xfer([00],300000,20)
#        time.sleep(.0001)
        if (dummy[0] != 0):
            num = dummy[0]
            id = id + chr(num)
            count = count + 1
        else:
            count=20
    GPIO.output(ppFRAME,False)
    return id	
	
def getPROGdata(addr,paddr):	#read a byte of data from program memory
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF0,paddr>>8,paddr&0xFF,2)
    value=(256*resp[0]+resp[1])
    return hex(value)	

def Poll():
    ppFoundCount=0
    for i in range (0,8):
        rtn = getADDR(i)
        if ((rtn-8)==i):
            print ("DAQCplate found at address",rtn-8)
            ppFoundCount += 1
    if (ppFoundCount == 0):
        print ("No DAQCplates found")

def VerifyDINchannel(din):
    assert ((din>=0) and (din<=7)),"Digital input channel value out of range. Must be in the range of 0 to 7"

def VerifyAINchannel(ain):
    assert ((ain>=0) and (ain<=8)),"Analog input channel value out of range. Must be in the range of 0 to 8"    
   
def VerifyDOUTchannel(dout):
    assert ((dout>=0) and (dout<=6)),"Digital output channel value out of range. Must be in the range of 0 to 6"
   
def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"DAQCplate address must be in the range of 0 to 7"
    addr_str=str(addr)
    assert (daqcsPresent[addr]==1),"No DAQCplate found at address "+addr_str
	
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global GPIObaseADDR
    arg = list(range(4))
    resp = []
    arg[0]=addr+GPIObaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    #    time.sleep(.0005)
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,40)
    #null = spi.writebytes(arg)   
    if bytes2return>0:
        time.sleep(.0001)        
        for i in range(0,bytes2return):	
            dummy=spi.xfer([00],500000,20)
            resp.append(dummy[0])        
    GPIO.output(ppFRAME,False)
    time.sleep(.0003)
    return resp	
	

def Init():	
    global daqcsPresent
    global Vcc
    for i in range (0,8):
        daqcsPresent[i]=0
        Vcc[i]=10000
        rtn = getADDR(i)
        if ((rtn-8)==i):
            daqcsPresent[i]=1
            ok=0
            while(ok==0):              
                Vcc[i] = getADC(i,8)
                if Vcc[i]>3.0:
                    ok=1
            setDOUTall(i,0)
            setPWM(i,0,0)
            setPWM(i,1,0)

    
Init()



import spidev
import time
import string
import site
import sys
import math
from numbers import Number
import RPi.GPIO as GPIO
from six.moves import input as raw_input
GPIO.setwarnings(False)

#Initialize
if (sys.version_info < (2,7,0)):
    sys.stderr.write("You need at least python 2.7.0 to use this library")
    exit(1)
    
GPIO.setmode(GPIO.BCM)
THERMObaseADDR=40
ppFRAME = 25
ppINT = 22
ppACK = 23
GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)    #Initialize SRQ input and ACK
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
spi = spidev.SpiDev()
spi.open(0,1)	
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/THERMOhelp.txt'
#helpPath='THERMOhelp.txt'       #for development only
THERMOversion=1.3
#1.0 - initial release
#1.1 - added line frequency options
#1.2 - added data smoothing options
#1.3 - fixed coefficients in type K conversion polynomial for T>500C. 

DataGood=False
#tType='k'   #Default thermocouple is type K

RMAX = 2000
MAXADDR=8
kTc=[[0 for z in range(3)] for x in range(10)]    #Type K thermocouple coefficients. Found at: https://www.keysight.com/upload/cmc_upload/All/5306OSKR-MXD-5501-040107_2.htm?&amp&cc=HK&lc=eng
#kTc[n]=[c0, c1, c2, c3, c4, c5, c6, c7, c8, c9]
kTc[0]=[0,2.5173462E01,-1.1662878,-1.0833638,-8.9773540E-01,-3.7342377E-01,-8.6632643E-02,-1.0450598E-02,-5.1920577E-04,0]
kTc[1]=[0,2.508355E01,7.860106E-02,-2.503131E-01,8.315270E-02,-1.228034E-02,9.804036E-04,-4.413030E-05,1.0577340E-06,-1.052755E-08]
kTc[2]=[-1.318058E02,4.830222E01,-1.646031,5.464731E-02,-9.650715E-04,8.802193E-06,-3.110810E-08,0,0,0]
# Voltage:	            -5.891 mV to 0 mV   0 mV to 20.644 mV   20.644 to 54.886mV
# Temperature:	            -200C to 0C	        0C to 500C	       500C to 1372C
# Coefficient Index:              0                1                    3

#Coefficients to convert temperature to Type K voltage (in millivolts)
kTcj=[-1.7600413686E-02, 3.8921204975E-02, 1.8558770032E-05, -9.9457592874E-08, 3.1840945719E-10, -5.6072844889E-13, 5.6075059059E-16, -3.2020720003E-19, 9.7151147152E-23, -1.2104721275E-26]

jTc=[[0 for z in range(3)] for x in range(9)]    #Type J thermocouple coefficients. Found at: https://www.keysight.com/upload/cmc_upload/All/5306OSKR-MXD-5501-040106_2.htm
#jTc[n]=[c0, c1, c2, c3, c4, c5, c6, c7, c8]
jTc[0]=[0,1.9528268E1,-1.2286185,-1.0752178,-5.9086933E-01,-1.7256713E-01,-2.8131513E-02,-2.3963370E-03,-8.3823321E-05]
jTc[1]=[0,1.978425E01,-2.001204E-01,1.036969E-02,-2.549687E-04,3.585153E-06,-5.344285E-08,5.099890E-10,0]
jTc[2]=[-3.1135818702E03,3.00543684E02,-9.94773230,1.70276630E-01,-1.43033468E-03,4.73886084E-06,0,0,0]
# Voltage:	            -8.095 mV to 0 mV   0 mV to 42.919 mV   42.919 to 54.00mV
# Temperature:	            -210C to 0C	        0C to 760C	       760C to 934C
# Coefficient Index:              0                1                    3

#Coefficients to convert temperature to Type J voltage (in millivolts)
jTcj=[0, 5.0381187815E-02, 3.0475836930E-05, -8.5681065720E-08, 1.3228195295E-10, -1.7052958337E-13, 2.0948090697E-16, -1.2538395336E-19, 1.5631725697E-23]


#Global Declarations
THERMOsPresent = list(range(8))
calScale=[[0 for z in range(8)] for x in range(8)]   #24 bit floating point slope calibration values
calOffset=[[0 for z in range(8)] for x in range(8)]  #24 bit floating point offset calibration values
calBias=[0,0,0,0,0,0,0,0]
calSet=list(range(8))
tempScale='c'

tType=[['k' for z in range(8)] for x in range(8)] #Default thermocouple is type K

def CLOSE():
	spi.close()
	GPIO.cleanup()

def Help():
	help()

def HELP():
	help()	
	
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")
 
def getTEMP(addr,channel,scale=None):
    global tempScale
    VerifyADDR(addr)
    assert ((channel>=1) and (channel<=12)),"Channel value out of range. Must be a value between 1 and 12"
    if scale is None:
        scal=tempScale
    else:
        scal=scale.lower()
        assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    channel-=1;
    Tvals=[[0],[0]]
    resp=ppCMD(addr,0x70,channel,0,4)   #initiate measurement
    Tvals[0]=resp[0]*256+resp[1]        #T channel data
    Tvals[1]=resp[2]*256+resp[3]        #Cold junction value
    if channel>7:
        Temp=Tvals[0]
        if (Temp>0x8000):
            Temp = Temp^0xFFFF
            Temp = -(Temp+1)
        Temp = Temp/16.0
        if (scal=='k'):
            Temp = Temp + 273.15
        if (scal=='f'):
            Temp = Temp*1.8+32.0
    else:
        CJtemp=Tvals[1]*2400.0/65535.0  #convert cold junction reading to voltage
        CJtemp=(10.888-math.sqrt((10.888**2.0)+4*0.00347*(1777.3-CJtemp)))/(2*(-0.00347))+30.0  #convert cold junction voltage to temperature
        Vcj=0
        if (tType[addr][channel]=='k'):
            for i in range(10):
                Vcj+=kTcj[i]*(CJtemp**i)     #Convert cold junction temperature to Type K voltage
            #print Vcj
            #Convert thermocouple A2D measurement to voltage and apply calibration values
            Vmeas=((Tvals[0]*2.4/65535.0)-calOffset[addr][channel])/calScale[addr][channel]*1000   # convert thermocouple A/D value to a voltage (in millivolts)
            Vhot=Vmeas+Vcj-calBias[addr]*1000.0 #Add cold junction voltage and subtract Vbias from measured voltage

            k=1
            if (Vhot<0):
                k=0
            if (Vhot>20.644):
                k=2
            Temp=0
            for i in range(10):             #convert adjusted measured thermocouple voltage to temperature
                Temp+=kTc[k][i]*(Vhot**i) 
        else:
            for i in range(9):
                Vcj+=jTcj[i]*(CJtemp**i)     #Convert cold junction temperature to Type J voltage
            #Convert thermocouple A2D measurement to voltage and apply calibration values
            Vmeas=((Tvals[0]*2.4/65535.0)-calOffset[addr][channel])/calScale[addr][channel]*1000   # convert thermocouple A/D value to a voltage (in millivolts)
            Vhot=Vmeas+Vcj-calBias[addr]*1000.0 #Add cold junction voltage and subtract Vbias from measured voltage
            #print Vhot, Vmeas, Vcj, calBias[addr]
            k=1
            if (Vhot<0):
                k=0
            if (Vhot>42.919):
                k=2
            Temp=0
            
            for i in range(9):             #convert adjusted measured thermocouple voltage to temperature
                Temp+=jTc[k][i]*(Vhot**i)    
                #print k, jTc[k][i], Temp
        if scal!='c':
            if scal=='f':
                Temp=Temp*1.8+32.0
            else:
                Temp+=273.15
    Temp=round(Temp,3)
    return Temp

def getCOLD(addr,scale=None):
    global tempScale
    VerifyADDR(addr)
    if scale is None:
        scal=tempScale
    else:
        scal=scale.lower()
        assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    channel=0;
    Tvals=[[0],[0]]
    resp=ppCMD(addr,0x70,channel,0,4)   #initiate measurement
    Tvals[1]=resp[2]*256+resp[3]        #Cold junction value - discard thermocouple measurement
    CJtemp=Tvals[1]*2400.0/65535.0      #convert cold junction reading to voltage
    CJtemp=(10.888-math.sqrt((10.888**2.0)+4*0.00347*(1777.3-CJtemp)))/(2*(-0.00347))+30.0  #convert cold junction voltage to temperature
    if scal!='c':
        if scal=='f':
            CJtemp=CJtemp*1.8+32.0
        else:
            CJtemp+=273.15
    CJtemp=round(CJtemp,3)
    return CJtemp

def getRAW(addr,channel):
    global tType
    VerifyADDR(addr)
    assert ((channel>=1) and (channel<=8)),"Channel value out of range. Must be a value between 1 and 8"
    channel-=1
    Tvals=[[0],[0]]
    resp=ppCMD(addr,0x70,channel,0,4) #initiate measurement
    Tvals[0]=resp[0]*256+resp[1]        #T channel data
    Tvals[1]=resp[2]*256+resp[3]        #Cold junction value    
    Vmeas=((Tvals[0]*2.4/65535.0)-calOffset[addr][channel])/calScale[addr][channel]*1000   # convert thermocouple A/D value to a voltage (in millivolts)
    Vraw=Vmeas-calBias[addr]*1000.0 #subtract Vbias from measured voltage   
    #print Tvals[0]*2.4/65535.0, Vmeas, calBias[addr]*1000.0
    return Vraw
    
def setSCALE(scale):
    global tempScale
    scal=scale.lower()
    assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    tempScale=scal
    
def getSCALE():
    global tempScale
    return tempScale
    
def setTYPE(addr,chan,type):
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=8)),"Thermocouple channel value out of range. Must be a value between 1 and 8"
    type=type.lower()
    assert ((type=='k') or (type=='j')), "Thermocouple type must be 'k' or 'j'"
    tType[addr][chan-1]=type 
    
def getTYPE(addr,chan):
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=8)),"Thermocouple channel value out of range. Must be a value between 1 and 8"
    return tType[addr][chan-1]
    
    
def setLINEFREQ(addr,freq):
    VerifyADDR(addr)
    assert ((freq==50) or (freq==60)),"Frequency value out of range. Must be a either 50 or 60"
    resp=ppCMD(addr,0x73,freq,0,0)
    
def setSMOOTH(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x74,1,0,0)
    
def clrSMOOTH(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x74,0,0,0)    

#===============================================================================#	
# Interrupt Functions	                                                   		    #
#===============================================================================#	
def setINTchannel(addr, channel):	# enable interrupt at end of temperature measurement
    VerifyADDR(addr)
    assert ((channel>=1) and (channel<=12)),"Channel value out of range. Must be a value between 1 and 12"	
    channel-=1;
    resp=ppCMD(addr,0x71,channel,0,0)

def intEnable(addr):	#THERMOplate will pull down on INT pin if an enabled event occurs
    VerifyADDR(addr)
    resp=ppCMD(addr,0x04,0,0,0)
	
def intDisable(addr):   #THERMOplate will not assert interrupts
    VerifyADDR(addr)
    resp=ppCMD(addr,0x05,0,0,0)
    
def getINTflags(addr):	#read INT flag register in THERMOplate - this clears interrupt line and the register
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,1)
    return resp[0]
    
#===============================================================================#	
# LED Functions	                                                   		   		#
#===============================================================================#			
def setLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x62,0,0,0)	

def getLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x63,0,0,1)
    return resp[0]	

        
#==============================================================================#	
# System Functions	                                                           #
#==============================================================================#
def getFWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0
	
def getHWrev(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0	

def getVersion():
    return THERMOversion   

def setINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF5,0,0,0)
	
def getID(addr):
    global THERMObaseADDR
    VerifyADDR(addr)
    addr=addr+THERMObaseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True): 
        ppFRAME = 25
        GPIO.output(ppFRAME,True)
        null=spi.xfer(arg,500000,50)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):              
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            count=0 
            csum=0
            go=True
            while (go): 
                dummy=spi.xfer([00],500000,40)
                if (dummy[0] != 0):
                    num = dummy[0]
                    csum += num
                    id = id + chr(num)
                    count += 1
                else:
                    dummy=spi.xfer([00],500000,40)  
                    checkSum=dummy[0]                
                    go=False 
                if (count>25):
                    go=False
                    DataGood=False
            if ((~checkSum & 0xFF) != (csum & 0xFF)):
                DataGood=False
        GPIO.output(ppFRAME,False)
    return id   
 
 
#==============================================================================#
# Flash Memory Functions - used for calibration constants                      #
#==============================================================================#
def CalGetByte(addr,ptr):
    VerifyADDR(addr)
    assert ((ptr>=0) and (ptr<=255)),"Calibration pointer is out of range. Must be in the range of 0 to 255" 
    resp=ppCMD(addr,0xFD,2,ptr,1)
    return resp[0]
    
def CalPutByte(addr,data):
    VerifyADDR(addr)
    assert ((data>=0) and (data<=255)),"Calibration value is out of range. Must be in the range of 0 to 255"
    resp=ppCMD(addr,0xFD,1,data,0)
    
def CalEraseBlock(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xFD,0,0,0)
    
#==============================================================================#	
# LOW Level Functions	                                                       #
#==============================================================================#          
def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"THERMOplate address out of range"
    addr_str=str(addr)
    assert (THERMOsPresent[addr]==1),"No THERMOplate found at address "+addr_str
    
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global THERMObaseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr+THERMObaseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    t0=time.time()
    wait=True    
    while(wait):
        if (GPIO.input(ppACK)==1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        ppFRAME = 25    
        GPIO.output(ppFRAME,True)
        null=spi.xfer(arg,500000,5)
        #DataGood=True
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.05):   #timeout
                wait=False
                DataGood=False    
        if (bytes2return>0) and DataGood:
            t0=time.time()
            wait=True
            while(wait):
                if (GPIO.input(ppACK)!=1):              
                    wait=False
                if ((time.time()-t0)>0.08):   #timeout
                    wait=False
                    DataGood=False
            if (DataGood==True):
                for i in range(0,bytes2return+1):	
                    dummy=spi.xfer([00],500000,5)
                    resp.append(dummy[0])
                csum=0;
                for i in range(0,bytes2return):
                    csum+=resp[i]
                if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                    DataGood=False
        GPIO.output(ppFRAME,False)
    return resp

def verifyTC(addr,channel):
    VerifyADDR(addr)
    assert ((channel>=0) and (channel<=7)),"Channel value out of range. Must be a value between 1 and 13"
    scal=scale.lower()
    Tvals=[[0],[0]]
    resp=ppCMD(addr,0x70,channel,0,4)   #initiate measurement
    Tvals[0]=resp[0]*256+resp[1]        #T channel data
    Tvals[1]=resp[2]*256+resp[3]        #Cold junction value
    #Convert thermocouple measurement to voltage and apply calibration
    Vmeas=((calScale[addr][channel]*Tvals[0]*2.4/65535.0)/37.5-(0.01*calOffset[addr][channel]))*1000   # convert thermocouple A/D value to a voltage (in millivolts)
    return Vmeas
    
def getADDR(addr):
    global THERMObaseADDR
    resp=ppCMD(addr,0x00,0,0,1)
    if (DataGood):
        return resp[0]-THERMObaseADDR
    else:
        return 8
    
def quietPoll():   
    global THERMOsPresent
    ppFoundCount=0
    for i in range (0,8):
        THERMOsPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            THERMOsPresent[i]=1
            ppFoundCount += 1
            getCalVals(i)
            #RESET(i)

#Function to convert from binary 32bit number to floating point            
def Binary2Cal(valList):
    polarity=1
    expsign=1
    val=valList[0]
    for i in range(3):
        val = val << 8
        val += valList[i+1]
    if (val & (2**31)) != 0:
        polarity = -1
    exp=((val>>24)&0x7F)-64
    if (exp<0):
        expsign=-1
    val=val&((2**24)-1)
    frac=float(val)/float(((2**24)-1))*expsign
    CalVal=math.pow(10,exp+frac)*polarity
    return CalVal                     
            
# Function to pull calibration data from flash memory
# data consists of 17 signed 32 bit floating point numbers stored as:
# |sign|7 bit exponent (-64 to 63)|24 bit mantissa normalized to (2^24-1)|
# Data is saved as Bias Voltage, offset0, slope0, offset1, slope1 ...offset7, slope7      
def getCalVals(addr):
    global calScale
    global calOffset
    global calBias
    values=list(range(4))
    for j in range(4):
        values[j]=CalGetByte(addr,j)    #get bias voltage
    calBias[addr]=Binary2Cal(values)
    for i in range(8):
        for j in range(4):
            values[j]=CalGetByte(addr,8*i+j+4)
        calOffset[addr][i]=Binary2Cal(values)
        for j in range(4,8):
            values[j-4]=CalGetByte(addr,8*i+j+4)          
        calScale[addr][i]=Binary2Cal(values)        
    
def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)
    time.sleep(.10)

quietPoll()


import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *
from DIO  import *
from PWM import *

#===============================================================================#	
# LED Functions	                                                   		   		#
#===============================================================================#			
def setLED(addr,chan,bright=None):
    assert ((chan>=0) and (chan<=8)),"Invalid LED number - must be between 0 and 8"
    VerifyADDR(addr)
    if (chan==0):
        assert (bright==None),"The brightness can only be controlled on channels 1 through 6."
        resp=ppCMD(addr,0x60,0,0,0)
    else:
        if (bright==None):
            resp=ppCMD(addr,0x90,chan-1,1,0)    #force DOUT mode
            clrDOUT(addr,chan)
        else:
            assert ((chan>=1) and (chan<=6)),"The brightness can only be controlled on channels 1 through 6."
            assert ((bright>=0.0) and (bright<=100.0)), 'Brightness must be between 0 and 100%'
            resp=ppCMD(addr,0x90,chan-1,3,0)    #force PWM mode
            bright=100.0-bright
            setPWM(addr,chan,bright)

def clrLED(addr,chan):
    assert ((chan>=0) and (chan<=8)),"Invalid LED number - must be between 0 and 8"
    VerifyADDR(addr)
    if (chan==0):
        resp=ppCMD(addr,0x61,0,0,0)
    else:
        resp=ppCMD(addr,0x90,chan-1,1,0)    #force DOUT mode
        setDOUT(addr,chan)
        #setPWM(addr,chan,100.0)

def toggleLED(addr,chan):
    assert ((chan>=0) and (chan<=8)),"Invalid LED number - must be between 0 and 8"
    VerifyADDR(addr)
    if (chan==0):
        resp=ppCMD(addr,0x62,0,0,0)	
    else:
        state=getDIN(addr,chan)
        resp=ppCMD(addr,0x90,chan-1,1,0)    #force DOUT mode
        if (state==0):
            setDOUT(addr,chan)
        else:
            clrDOUT(addr,chan)      

def getLED(addr,chan):
    assert ((chan>=0) and (chan<=8)),"Invalid LED number - must be between 0 and 8"
    VerifyADDR(addr)
    if (chan==0):    
        resp=ppCMD(addr,0x63,0,0,1)
        arg=resp[0]
    else:
        arg=getDIN(addr,chan)
        if (arg==0):
            arg=1
        else:
            arg=0
    return arg

def setRGB(addr,chan,red,grn,blu):
    assert ((chan>=1) and (chan<=8)),"Invalid LED channel - must be between 1 and 8"
    VerifyADDR(addr)
    VerifyCHAN(chan)
    ppCMD(addr,0x64,chan-1,0,0) #select the channel
    ledarg=RGBto24(red,grn,blu)
    ledarg=RGB24to565(ledarg)  #Convert 8:8:8 to 5:6:5
    param1=ledarg>>8
    param2=ledarg&0xFF    
    ppCMD(addr,0x65,param1,param2,0)    #send the data
    
def setRGBSTRING(addr,chan,string):
    assert ((chan>=1) and (chan<=8)),"Invalid LED channel - must be between 1 and 8"
    VerifyADDR(addr)
    VerifyCHAN(chan)
    assert (len(string)>=3), "LED string length is too sort. Must be 3 or more."
    assert (len(string)<=64*3), "LED string length is too long. Must be 3*64 or less."
    assert ((len(string)%3)==0), "LED string length must be a mulitple of 3."
    ppCMD(addr,0x64,chan-1,0,0) #set the channel
    ledarg=RGBto24(string[0],string[1],string[2])  #Convert 8:8:8 to 5:6:5
    ledarg=RGB24to565(ledarg)
    param1=ledarg>>8
    param2=ledarg&0xFF
    resp=ppCMD(addr,0x66,param1,param2,0)     #Start LED string command
    if (len(string)>3):
        for i in range(len(string)//3-1):
            ledarg=RGBto24(string[(i+1)*3],string[(i+1)*3+1],string[(i+1)*3+2])
            ledarg=RGB24to565(ledarg)
            param1=ledarg>>8
            param2=ledarg&0xFF
            resp=ppCMD(addr,0x67,param1,param2,0)     #Append LED string command
    resp=ppCMD(addr,0x68,0,0,0)             #String complete. Write to LEDs.
            
def RGBto24(red,grn,blu):
    red=int(red)
    assert (red<=255), "Red color is greater than 255."
    grn=int(grn)
    assert (grn<=255), "Green color is greater than 255."
    blu=int(blu)
    assert (blu<=255), "Blue color is greater than 255."
    rgb24=(red<<16)+(grn<<8)+blu
    return(rgb24)   
    
def RGB24to565(val888):
    red=val888>>16
    lowbits=red&0x7
    red=red>>3
    if ((lowbits>3) and (red<31)):
        red += 1
    grn=(val888>>8)&0xFF
    lowbits=grn&0x3
    grn=grn>>2
    if ((lowbits>1) and (grn<63)):
        grn += 1   
    blu=val888&0xFF
    lowbits=blu&0x7
    blu=blu>>3
    if ((lowbits>3) and (blu<31)):
        blu += 1
    val565=(red<<11) + (grn<<5) + blu
    return(val565)
    


import os
import sys
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)
from BASE import *

#==============================================================================#	
# PWM Functions                                                                #
#==============================================================================# 
def setPWM(addr,chan,dutyCycle):
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=6)),"PWM port number is out of range. Must between 1 and 6"
    assert ((dutyCycle>=0)and(dutyCycle<=100)), "Duty Cycle must be a value between 0.0 and 100.0"
    registerVal=int(dutyCycle*1024.0/100.0+0.5)
    param1=((chan-1)<<4)+(registerVal>>8)
    param2=registerVal&0xFF
    resp=ppCMD(addr,0xC0,param1,param2,0)
    
